#!/usr/bin/env python3

import importlib.util
import os
import os.path
import shutil
import subprocess
import sys

this_dir = os.path.abspath(os.path.dirname(__file__) or ".")

def version(build_it):
    return "1.14.6"

def install(build_it):
    zip_ = "hdf5-1.14.6.tar.gz"
    src_dir = "hdf5-1.14.6"
    if os.path.isdir(src_dir):
        shutil.rmtree(src_dir)
    build_it.extract_file(zip_)
    os.chdir(src_dir)

    if not os.path.isdir('build'):
        os.makedirs('build')
    os.chdir('build')

    testingSwitch = "-DBUILD_TESTING=" + (os.environ['LIB3_CHECK'] == "1" and "1" or "0")
    fortranSwitch = "-DHDF5_BUILD_FORTRAN=" + (os.environ['OCSSW_ARCH'] == "macosx_intel" and "0" or "1")
    buildType = "-DCMAKE_BUILD_TYPE=Release"

    # fix the LDFLAGS for configure
    os.environ['LDFLAGS'] = os.environ['LDFLAGS_cmake']

    cmd = ["cmake", "..", "-DCMAKE_PREFIX_PATH=%s" % (build_it.opt_dir),"-DHDF5_ENABLE_ROS3_VFD=ON", "-DCMAKE_INSTALL_PREFIX=%s" % (build_it.opt_dir), "-DHDF5_BUILD_CPP_LIB=0", "-DHDF5_BUILD_EXAMPLES=0", "-DHDF5_BUILD_HL_LIB=1", "-DHDF5_DISABLE_COMPILER_WARNINGS=1", "-DHDF5_ENABLE_Z_LIB_SUPPORT=1", fortranSwitch, buildType, testingSwitch]
    #if os.environ.get('OCSSW_MPI') == '1':
    #    cmd.append("-DHDF5_ENABLE_PARALLEL=ON")

    subprocess.check_call(cmd)
    subprocess.check_call(["cmake", "--build", ".", "--config", "Release"])
    subprocess.check_call(["cmake", "--build", ".", "--target", "install"])

    os.chdir(this_dir)

    if not os.environ['OCSSW_DEBUG'] or os.environ['OCSSW_DEBUG'] == "0":
        shutil.rmtree(src_dir)

if __name__ == "__main__":
    os.chdir(this_dir)

    spec = importlib.util.spec_from_file_location("BuildIt", os.path.join("..", "BuildIt.py"))
    build_it = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(build_it)
    master = build_it.BuildIt()
    master.setup_environment()
    sys.exit(install(master))
