#!/usr/bin/env python3

import importlib.util
import os
import os.path
import shutil
import subprocess
import sys

this_dir = os.path.abspath(os.path.dirname(__file__) or ".")

def version(build_it):
    return "4.9.3_oel 4.3.1 4.6.1"

def install(build_it):
    os.environ['LDFLAGS'] = "-L%s/lib %s" % (build_it.opt_dir, os.environ['LDFLAGS'])
    os.environ['CFLAGS'] = "-I%s/include %s" % (build_it.opt_dir, os.environ['CFLAGS'])
    os.environ['LD_LIBRARY_PATH'] = "%s/lib" % (build_it.opt_dir)

    # os.environ['LDFLAGS'] = os.environ['LDFLAGS'].replace("$", "$$")
    # print("LDFLAGS = ***", os.environ['LDFLAGS'], "***")
    # exit(1)


    zip_ = "netcdf_oel_c.tar.gz"
    src_dir = "netcdf-c"
    if os.path.isdir(src_dir):
        shutil.rmtree(src_dir)
    build_it.extract_file(zip_)
    os.chdir(src_dir)

    os.mkdir("build")
    os.chdir("build")

    # fix the LDFLAGS for configure
    if os.environ['OCSSW_ARCH'] == "macosx_arm64":
        os.environ["LDFLAGS"] = ""
    else:
        os.environ['LDFLAGS'] = os.environ['LDFLAGS_cmake']
    #cmd = ["./configure", "--prefix=%s" % (build_it.opt_dir), "--disable-dap", "--enable-netcdf-4", "--enable-hdf4"]
    cmd = ["cmake", "..","-DCMAKE_BUILD_TYPE=Release", "-DNETCDF_ENABLE_DAP=ON","-DNETCDF_ENABLE_BYTERANGE=ON","-DNETCDF_ENABLE_HDF5_ROS3=ON", "-DNETCDF_ENABLE_HDF5=ON",  "-DNETCDF_ENABLE_HDF4=ON",  "-DCMAKE_INSTALL_PREFIX=" + build_it.opt_dir, "-DCMAKE_INSTALL_LIBDIR=lib", "-DCMAKE_PREFIX_PATH=" +build_it.opt_dir]

    subprocess.check_call(cmd)
    subprocess.check_call(["make"])
    #subprocess.check_call(["make", "test"])
    subprocess.check_call(["make", "install"])

    os.chdir(this_dir)

    if not os.environ['OCSSW_DEBUG'] or os.environ['OCSSW_DEBUG'] == "0":
        shutil.rmtree(src_dir)

    os.environ['LIBS'] = "-lnetcdf -lhdf5"

    zip_ = "netcdf_oel_cpp.tar.gz"
    src_dir = "netcdf-cxx4"
    if os.path.isdir(src_dir):
        shutil.rmtree(src_dir)
    build_it.extract_file(zip_)
    os.chdir(src_dir)
    patch_dir = os.path.join(this_dir, "patches")
    with open(os.path.join(patch_dir, "ncGroup.h.patch"), 'r') as patch_h:
        subprocess.check_call(["patch", "-b", "-p0"], stdin=patch_h)
    # subprocess.check_call(["autoupdate"])
    # subprocess.check_call(["autoreconf","-if"])
   # subprocess.check_call(["mkdir","build"])
   # os.chdir("build")
    os.mkdir("build")
    os.chdir("build")

    subprocess.check_call(["cmake","..","-DNCXX_ENABLE_TESTS=OFF","-DCMAKE_INSTALL_PREFIX=" + build_it.opt_dir, "-DCMAKE_PREFIX_PATH=" +build_it.opt_dir,"-DCMAKE_INSTALL_LIBDIR=lib", "-DCMAKE_CXX_FLAGS=-O3 -std=c++14 -I%s/include" % (build_it.opt_dir)])
    subprocess.check_call(["make"])
    #subprocess.check_call(["make", "test"])
    subprocess.check_call(["make", "install"])

    os.chdir(this_dir)

    if not os.environ['OCSSW_DEBUG'] or os.environ['OCSSW_DEBUG'] == "0":
        shutil.rmtree(src_dir)

    zip_ = "netcdf-fortran-4.6.1.tar.gz"
    src_dir = "netcdf-fortran-4.6.1"
    if os.path.isdir(src_dir):
        shutil.rmtree(src_dir)
    build_it.extract_file(zip_)
    os.chdir(src_dir)

    # fix the LDFLAGS for configure
    os.environ['LDFLAGS'] = os.environ['LDFLAGS_configure']

    cmd = ["./configure", "--prefix=%s" % (build_it.opt_dir)]
    #if os.environ.get('OCSSW_MPI') == '1':
    #    cmd.append("--enable-parallel-tests")
    #    os.environ["FC"] = "mpifort"
    subprocess.check_call(cmd)
    subprocess.check_call(["make"])
    #subprocess.check_call(["make", "test"])
    subprocess.check_call(["make", "install"])

    os.chdir(this_dir)

    if not os.environ['OCSSW_DEBUG'] or os.environ['OCSSW_DEBUG'] == "0":
        shutil.rmtree(src_dir)


if __name__ == "__main__":
    os.chdir(this_dir)

    spec = importlib.util.spec_from_file_location("BuildIt", os.path.join("..", "BuildIt.py"))
    build_it = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(build_it)
    master = build_it.BuildIt()
    master.setup_environment()
    sys.exit(install(master))
