#!/usr/bin/env python3

import time
import importlib.util
import os
import os.path
import shutil
import subprocess
import sys

this_dir = os.path.abspath(os.path.dirname(__file__) or ".")

def version(build_it):
    return "5.2.20v1.01"

def install(build_it):
    zip_ = "SDPTK5.2.20v1.01.tar.Z"
    src_dir = "TOOLKIT"
    if os.path.isdir(src_dir):
        shutil.rmtree(src_dir)
    build_it.extract_file(zip_)
    patch_dir = os.path.join(this_dir, "patches")
    with open(os.path.join(patch_dir, "SDPTK5.2.20v1.01.patch"), 'r') as patch_h:
        subprocess.check_call(["patch", "-d", src_dir, "-p1"], stdin=patch_h)
    with open(os.path.join(patch_dir, "INSTALL.patch"), 'r') as patch_h:
        subprocess.check_call(["patch", "-d", src_dir, "-p1"], stdin=patch_h)
    with open(os.path.join(patch_dir, "PGS_EPH_EphemAttit.c.patch"), 'r') as patch_h:
        subprocess.check_call(["patch", "-d", src_dir, "-p0"], stdin=patch_h)
    with open(os.path.join(patch_dir, "PGS_EPH_EphAtt_unInterpolate.c.patch"), 'r') as patch_h:
        subprocess.check_call(["patch", "-d", src_dir, "-p0"], stdin=patch_h)
    with open(os.path.join(patch_dir, "CUC_UDUNITS_limits.h.patch"), 'r') as patch_h:
        subprocess.check_call(["patch", "-d", src_dir, "-p0"], stdin=patch_h)

    os.chdir(src_dir)

    # FIX for HDF-4.2.16, libdf is now combined with libhdf
    makefiles = ['src/EPH/makefile', 'src/EPH/makefile.cpp', 'src/EPH/orbsim/makefile', 'src/EPH/orbsim/makefile.cpp', 'src/EPH/gbadsim/makefile', 'src/EPH/gbadsim/makefile.cpp']
    for m in makefiles:
        subprocess.check_call(['sed', '-i', '-e', 's/ldf/lhdf/', m])

    install_dir = os.path.join(build_it.opt_dir, 'EOS')
    eos_prefix = os.environ['EOS_LIB_PREFIX']

    bin_path = os.path.join(install_dir, 'bin', eos_prefix)

    if eos_prefix == "macarm64":
        lib_path = os.path.join(install_dir, 'lib/macintosh')
        bin_path = os.path.join(install_dir, 'bin/macintosh')
        if not os.path.exists(lib_path):
            os.symlink(os.path.join(install_dir, 'lib', "macarm64"), lib_path)

    f77 = shutil.which(os.environ['F77'])
    cc  = shutil.which(os.environ['CC'])
    cpp = shutil.which(os.environ['CXX'])
    cmd = ["bin/INSTALL"]
    if os.environ['OCSSW_DEBUG'] == "1":
        cmd.append("-dbug")
    cmd.extend(("-notext","-batch", "-fc_path", f77, "-cc_path", cc)) #, "-cpp_path", cpp))
    cmd.extend(("-hdfhome", build_it.opt_dir))
    cmd.extend(("-hdfeos_home", install_dir))
    cmd.extend(("-hdf5home", build_it.opt_dir))
    cmd.extend(("-hdfeos5_home", install_dir))
    cmd.extend(("-zlibhome", build_it.opt_dir))
    cmd.extend(("-jpeghome", build_it.opt_dir))
    cmd.extend(("-sziphome", build_it.opt_dir))
    if os.environ['CC'] == "clang": 	
    	cmd.extend(("-cc_flags","-Wno-implicit-int -Wno-int-conversion -Wno-implicit-function-declaration"))	
    print(" ".join(cmd))
    subprocess.check_call(cmd)

    now = int(time.time())
    for d in ("bin", "lib"):
        for root, dirs, files in os.walk(d):
            for name in files:
                src = os.path.join(root, name)
                if now - os.path.getmtime(src) < 3600:
                    dest_dir = os.path.join(install_dir, root)
                    if not os.path.exists(dest_dir) and not os.path.isdir(dest_dir) and not os.path.islink(dest_dir):
                        os.makedirs(dest_dir)
                    shutil.copy(src, dest_dir)

    for root, dirs, files in os.walk('include'):
        for name in files:
            if name.endswith('.h'):
                dest_dir = os.path.join(install_dir, root)
                if not os.path.exists(dest_dir) and not os.path.isdir(dest_dir) and not os.path.islink(dest_dir):
                    os.makedirs(dest_dir)
                shutil.copy(os.path.join(root, name), dest_dir)

    for d in ("chkeph", "ephtobin", "PGS_CSC_UT1_update", "PGS_TD_NewLeap"):
        shutil.copy(os.path.join(bin_path, d), os.path.join(build_it.opt_dir, 'bin'))


    os.chdir(this_dir)

    if not os.environ['OCSSW_DEBUG'] or os.environ['OCSSW_DEBUG'] == "0":
        shutil.rmtree(src_dir)

if __name__ == "__main__":
    os.chdir(this_dir)

    spec = importlib.util.spec_from_file_location("BuildIt", os.path.join("..", "BuildIt.py"))
    build_it = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(build_it)
    master = build_it.BuildIt()
    master.setup_environment()
    sys.exit(install(master))
